
# q-expansion of newform 3762.2.a.bb, downloaded from the LMFDB on 21 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3762
weight = 2
poly_data = [-3, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, 0], [0, 0], [0, 1], [-1, 1], [-1, 0], [-1, -1], [0, 2], [1, 0], [6, -2], [6, 1], [-1, -1], [-4, 4], [3, -3], [8, -3], [6, 0], [6, 2], [0, 0], [2, 4], [5, -5], [12, -1], [2, -2], [2, 6], [-12, -1], [0, -2], [8, 0], [6, -6], [-4, 1], [6, -4], [2, 0], [0, 0], [-10, 2], [-3, -7], [15, -5], [-4, -1], [0, 0], [2, -8], [8, -3], [-16, 2], [-6, 8], [-6, 11], [-3, 7], [8, -10], [6, 0], [-4, -9], [6, -12], [-16, 6], [20, 2], [-16, -4], [12, -6], [5, 3], [0, -4], [-6, 13], [20, -3], [-24, 0], [0, 2], [18, 1], [6, 6], [5, -5], [-10, 0], [12, -5], [11, -13], [-9, 15], [-16, 6], [-12, -6], [2, 7], [0, 6], [2, 9], [-7, -1], [-12, 4], [14, 8], [6, 4], [21, -5], [2, -4], [14, -3], [-25, 5], [-15, 5], [0, -15], [11, -19], [-18, -2], [-16, 17], [-18, 4], [2, -14], [-12, 4], [8, -14], [-16, -4], [-12, -12], [18, 6], [8, 6], [12, -8], [8, -4], [18, 2], [-15, -9], [-4, -11], [-15, 13], [-34, 0], [-6, -7], [0, 8], [-18, -8], [-4, 10], [-10, 6], [-4, 0], [0, 12], [6, -12], [-33, 9], [23, -13], [-25, 3], [-12, 0], [42, 0], [36, -7], [11, -3], [-10, 6], [2, 0], [-9, 3], [2, -16], [-22, -10], [12, -14], [8, 8], [-12, -6], [3, -11], [-6, -10], [-4, 12], [-1, 1], [6, 1], [36, -8], [14, 8], [0, 12], [17, -1], [12, 6], [-28, 12], [2, -4], [8, -21], [-30, 12], [-4, -16], [-4, 15], [12, -14], [-10, -12], [-30, -8], [8, -10], [-12, -2], [-24, -4], [2, 12], [30, 2], [32, 0], [24, -20], [-46, 8], [-36, 7], [-28, -8], [9, 15], [-28, 16], [-3, 9], [-10, 25], [27, 3], [-16, -10], [12, -16], [32, -16], [0, -12], [-10, 7], [-3, 5], [38, -16], [30, 12], [-12, -6], [-6, 12], [-4, 12], [-6, 13], [0, -12], [-21, -1], [-19, -3], [14, -6], [23, 1], [-33, 13], [0, 24], [-16, 22], [-30, 9], [20, 2], [8, 0], [-6, 15], [17, 5], [-6, -16], [-22, 8], [-4, 6], [8, 12], [0, -4], [2, 20], [-12, -7], [24, 16], [18, 0], [-10, -4], [26, -9], [8, 19], [-6, 15], [14, -30], [39, -5], [14, -17], [0, -2], [57, -3], [6, -18], [8, -8], [35, 13], [-6, -4], [0, 24], [21, -11], [-28, 12], [8, 4], [20, -9], [-12, -18], [6, -24], [-34, 0], [18, -22], [36, 0], [-58, -1], [-10, 2], [-36, 7], [-28, 6], [63, -3], [-12, 12], [20, 6], [-4, -24], [30, 8], [3, 27], [-15, 27], [-49, 7], [2, -28], [24, -6], [26, -12], [12, -24], [26, -4], [-12, -22], [-36, 13], [-16, 10], [60, -1], [-4, 11], [26, 6], [-16, -22], [-27, -1], [62, 6], [27, -15], [5, 21], [42, 4], [-54, 0], [3, -9], [18, 3], [-13, 7], [-28, 10], [44, -4], [54, -8], [45, 3], [-64, 2], [-6, 19], [-22, -18], [-60, 12], [38, -12], [-12, -18], [6, -18], [26, 18], [-12, 0], [24, -5], [-40, -3], [-22, 38], [24, -38], [26, -8], [56, 6], [30, -22], [14, -8], [26, -13], [-54, 1], [-40, -12], [0, 0], [-6, -17], [8, -22], [-72, 6], [-10, 12], [-10, 11], [17, 9], [-34, 23], [44, 5], [-16, 10], [42, 10], [50, -7], [32, 5], [12, -35], [6, 0], [2, 32], [-54, 11], [62, -13], [-52, 17], [-18, 42], [11, 3], [9, -11], [26, 8], [-24, -1], [-3, 31], [36, 12], [-3, -21], [36, -2], [-46, 3], [-30, 0], [-19, 9], [6, -8], [-6, 0], [44, 7], [-22, 30], [21, 25], [-28, -20], [30, 15], [8, -36], [-64, -8], [9, 15], [56, -4], [12, -4], [20, 12], [-15, -21], [0, -39], [15, -3], [-22, 11], [33, -25], [-10, 8], [-42, 24], [42, -14], [2, -19], [6, -36], [-16, -30], [14, -32], [-6, 24], [8, 28], [30, 10], [20, 10], [-25, 21], [50, -5], [-60, -10], [12, 10], [56, 0], [24, 4], [-46, 9], [0, 40], [-28, -7], [-12, 18], [2, 10], [39, -19], [-34, 12], [74, -1], [20, 4], [30, 15], [33, -15], [68, 5], [24, -10], [18, -22], [35, 5], [5, -13], [6, 18], [-54, 12], [2, -36], [20, 30], [21, 19], [-34, 16], [-7, 17], [-48, -5], [-54, -10], [0, 29], [6, 20], [45, 21], [-22, 8], [-18, 22], [12, 15], [-6, -26], [26, 26], [-64, 0], [36, 24], [-16, 2], [-4, 19], [-18, 48], [-7, -21], [12, -12], [6, -7], [-28, 20], [14, -24], [-9, 9], [9, 3], [-28, 8], [6, 2], [26, -48], [24, 16], [27, 19], [-19, 35], [18, 6], [-16, -28], [0, -36], [44, 6], [44, -23], [23, 31], [0, -39], [14, -8], [54, -18], [0, 4], [2, 10], [-57, -9], [80, -5], [-40, -8], [-30, 48], [-7, -41], [-36, 4], [47, 23], [-6, 27], [-10, 40], [-48, 20], [-18, 17], [-52, 36], [-49, -19], [18, 0], [32, 0], [18, -24], [-10, -24], [-36, -6], [-12, 40], [44, -13], [62, -8], [-30, 4], [-48, -2], [-49, -1], [-27, -23], [-36, 10], [6, 32], [-82, 12], [-6, -14], [-18, 20], [-28, -12], [60, -16], [-27, 39], [24, -2], [44, -14], [-18, -15]]
